# SPDX-FileCopyrightText: 2023 spdx contributors
#
# SPDX-License-Identifier: Apache-2.0
from rdflib import Namespace

SPDX_NAMESPACE = Namespace("http://spdx.org/rdf/terms#")
POINTER_NAMESPACE = Namespace("http://www.w3.org/2009/pointers#")
REFERENCE_NAMESPACE = Namespace("http://spdx.org/rdf/references/")
LICENSE_NAMESPACE = Namespace("http://spdx.org/licenses/")
